// -----------------------------------------------------------------------------
// COPYRIGHT (C) 2020 - AUTHOR - MANISH JOSHI
//
// CREATED USING SCILAB-6.1.0 ON WINDOWS-10 64-bit
//
// DATE OF CREATION: 26 Oct, 2020 
//
// THIS FILE IS A PART OF LSF_Toolbox 
// -----------------------------------------------------------------------------


// THIS IS THE CALLBACK FUNCTION FOR POLYNOMIAL FIT BUTTON (Y = a1 + a2 * x + a3 * x^2 + ......)
function PolyFitBtn_Callback(handles)
	[x,y] =  Data()
	dx = (max(x)- min(x))/5
	X = linspace(min(x)-dx,max(x)+dx,1000)
	n = length ( x )
	o = evstr(handles.PolyOrderEntry.string)
	m = o+1
	if o == [] then
		messagebox("Please Enter the order of the polynomial", "Error!!","error")
		error("Please Enter the order of the polynomial")
	elseif n<=o 
		messagebox("Fitting is not possible please enter more data points ", "Error!!", "error")
		error("For polynomial fitting number of data points must be greater than order of polynomial")
	else
		for j = 1: m
			for k = 1: m
				C (j , k) = sum ( x .^( j+k -2) )
			end
			B ( j ) = sum( y .*( x .^( j -1 ) ) )
		end
		A = inv(C)*B;
		Y = poly(A, "X",  "coeff");
		S = sci2exp(Y);
		M = A\C;
		YY = poly(A, "x",  "coeff");
		YR = evstr(sci2exp(YY));
		Ymean = sum(YR)/n;
		SSR = sum((Ymean-YR).^2);
		SSE = sum((y-YR).^2);
		SST = SSE + SSR;
		Rsqr = SSR/SST;
		DFE = n-m;
		if n == m then
			ARsqr = %nan;
			RMSE = %nan;
		else
			ARsqr = 1-((1-Rsqr)*(n-1)/(n-o-1));
			RMSE = sqrt(SSE/(n-m))
		end
		plot(x,y,"k.",X,evstr(S),"r","thickness",3)
		if (o<=3)
			LegStr  = "Y = "+S;
		else
			LegStr  = "Poly"+ string(o);
		end
		LabelGraph(handles, LegStr, 2);
		ResultStr = "======= Polynomial Fitting =======\n\n"+...
		"coefficients of polynomial : \n   " + sci2exp(A') + "\n\n"+...
		"The Polynomial equation obtained is:\n  " + sci2exp(Y) + "\n\nGoodness of fit : \n " + ....
		msprintf("  SSE = %g ", SSE)+  "\n" + msprintf("   R^2= %g ", Rsqr)+  "\n" + msprintf("   DFE = %g ", DFE)+  "\n" + ....
		msprintf("   Adj. R^2 = %g ", ARsqr)+  "\n" + msprintf("   RMSE = %g ", RMSE)+  "\n" ;
		typ = "poly"+string(o);
		InsertTable(handles, typ, SSE,Rsqr, DFE, ARsqr, RMSE,  m)
		ResultString(handles,ResultStr);
	end
endfunction
